/* Anjuta
 * Copyright 2002 Jeroen Zwartepoorte
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 */

#include <config.h>
#include <pwd.h>
#include <bonobo.h>
#include <libanjuta/anjuta-tool.h>
#include <libanjuta/glue-plugin.h>
#include <libzvt/libzvt.h>

#define ZVT_FONT "-adobe-courier-medium-r-normal-*-*-120-*-*-m-*-iso8859-1"
#define ZVT_SCROLLSIZE 200

gpointer parent_class;
extern char **environ;

typedef struct {
	AnjutaTool parent;
} TerminalTool;

typedef struct {
	AnjutaToolClass parent;
} TerminalToolClass;

static void
new_terminal (GtkWidget *widget,
	      gpointer data)
{
}

static BonoboUIVerb verbs[] = {
	BONOBO_UI_UNSAFE_VERB ("NewTerminal", new_terminal),
	BONOBO_UI_VERB_END
};

static char **
get_child_environment (GtkWidget *term)
{
	/* code from gnome-terminal, sort of. */
	char **p;
	int i;
	char **retval;
#define EXTRA_ENV_VARS 6

	/* count env vars that are set */
	for (p = environ; *p; p++);

	i = p - environ;
	retval = g_new (char *, i + 1 + EXTRA_ENV_VARS);

	for (i = 0, p = environ; *p; p++) {
		/* Strip all these out, we'll replace some of them */
		if ((strncmp (*p, "COLUMNS=", 8) == 0) ||
		    (strncmp (*p, "LINES=", 6) == 0)   ||
		    (strncmp (*p, "TERM=", 5) == 0)    ||
		    (strncmp (*p, "GNOME_DESKTOP_ICON=", 19) == 0)) {
			/* nothing: do not copy */
		} else {
			retval[i] = g_strdup (*p);
			++i;
		}
	}

	retval[i] = g_strdup ("TERM=xterm"); /* FIXME configurable later? */
	++i;

	retval[i] = NULL;

	return retval;
}

static void
term_init_cb (GtkWidget *widget,
	      gpointer   data)
{
	ZvtTerm *term = ZVT_TERM (widget);
	struct passwd *pw;
	const char *shell;
	const char *name;
	char **env;

	zvt_term_reset (term, TRUE);
	switch (zvt_term_forkpty (term, ZVT_TERM_DO_UTMP_LOG | ZVT_TERM_DO_WTMP_LOG)) {
	case -1:
		break;
	case 0:
		pw = getpwuid (getuid ());
		if (pw) {
			shell = pw->pw_shell;
			name = "-";
		} else {
			shell = "/bin/sh";
			name = "-sh";
		}
		env = get_child_environment (widget);
		execle (shell, name, NULL, env);
		g_strfreev (env);
	default:
		break;
	}
}

static gboolean
term_focus_cb (GtkWidget *widget,
	       GdkEvent  *event,
	       gpointer   data)
{
	gtk_widget_grab_focus (widget);

	return TRUE;
}

static void
term_destroy_cb (GtkWidget *widget,
		 gpointer   data)
{
	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
					      G_CALLBACK (term_init_cb), NULL);
	zvt_term_closepty (ZVT_TERM (widget));
}

static void
shell_set (AnjutaTool *tool)
{
	GtkWidget *zvt, *sb, *frame, *hbox;
	BonoboControl *control;

	anjuta_tool_merge_ui (tool,
			      "anjuta-terminal-tool",
			      ANJUTA_DATADIR,
			      "anjuta-terminal-plugin.xml",
			      verbs,
			      tool);

	zvt = zvt_term_new ();
	zvt_term_set_font_name (ZVT_TERM (zvt), ZVT_FONT);
	zvt_term_set_blink (ZVT_TERM (zvt), FALSE);
	zvt_term_set_bell (ZVT_TERM (zvt), TRUE);
	zvt_term_set_scrollback (ZVT_TERM (zvt), ZVT_SCROLLSIZE);
	zvt_term_set_scroll_on_keystroke (ZVT_TERM (zvt), TRUE);
	zvt_term_set_scroll_on_output (ZVT_TERM (zvt), FALSE);
	zvt_term_set_background (ZVT_TERM (zvt), NULL, 0, 0);
	zvt_term_set_wordclass (ZVT_TERM (zvt), (unsigned char *) "-A-Za-z0-9,./?%&#:_");
	zvt_term_set_del_key_swap (ZVT_TERM (zvt), TRUE);
	zvt_term_set_del_is_del (ZVT_TERM (zvt), FALSE);

	g_signal_connect (G_OBJECT (zvt), "child_died",
			  G_CALLBACK (term_init_cb), tool);
	g_signal_connect(G_OBJECT (zvt), "button_press_event",
			 G_CALLBACK (term_focus_cb), tool);
	g_signal_connect (G_OBJECT (zvt), "destroy",
			  G_CALLBACK (term_destroy_cb), tool);

	sb = gtk_vscrollbar_new (GTK_ADJUSTMENT (ZVT_TERM (zvt)->adjustment));
	GTK_WIDGET_UNSET_FLAGS (sb, GTK_CAN_FOCUS);

	frame = gtk_frame_new (NULL);
	gtk_widget_show (frame);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), hbox);
	gtk_box_pack_start (GTK_BOX (hbox), zvt, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, TRUE, 0);
	gtk_widget_show_all (frame);

	control = bonobo_control_new (frame);

	anjuta_shell_add_control (tool->shell,
				  BONOBO_OBJREF (control),
				  "Terminal",
				  _("Terminal"),
				  NULL);

	term_init_cb (zvt, NULL);
}

static void
dispose (GObject *obj)
{
	anjuta_tool_unmerge_ui (ANJUTA_TOOL (obj));
}

static void
terminal_tool_instance_init (GObject *object)
{
}

static void
terminal_tool_class_init (GObjectClass *klass) 
{
	AnjutaToolClass *tool_class = ANJUTA_TOOL_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	tool_class->shell_set = shell_set;
	klass->dispose = dispose;
}

ANJUTA_TOOL_BOILERPLATE (TerminalTool, terminal_tool);

ANJUTA_SIMPLE_PLUGIN (TerminalTool, terminal_tool);
